-----------------------------------------------------------------------------
--
--	UART2DISPATCHER.VHDL
--
--
--	Author 	: Rene Bllhoff
--  Created : Feb. 2007
--
--	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
-----------------------------------------------------------------------------
--
--  This source file may be used and distributed without
--  restriction provided that this copyright statement is not
--  removed from the file and that any derivative work contains
--  the original copyright notice and the associated disclaimer.
--
--  This program is free software; you can redistribute it and/or
--  modify it under the terms of the GNU General Public License
--  as published by the Free Software Foundation; either version 2
--  of the License, or (at your option) any later version.
--										
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
--  General Public License for more details.
--
--  You should have received a copy of the GNU General Public
--  License along with this source; if not, download it	from
--  http://www.gnu.org/licenses/gpl.html	
--														
-----------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity uart2dispatcher is 
  port (clk 				: in 	std_logic;
        res 				: in 	std_logic;
        rx_in 			: in 	std_logic;
        tx_out			: out	std_logic;
				
				dp_out			: out std_logic_vector (7 downto 0);
				dp_in				: in  std_logic_vector (7 downto 0);
				dp_trg			: out std_logic;
				dp_sel			: out std_logic
			);
end uart2dispatcher;



architecture behave of uart2dispatcher is

	COMPONENT simple_uart
	PORT(
		clk : IN std_logic;
		res : IN std_logic;
		baudrate : IN std_logic_vector(15 downto 0);
		rx_in : IN std_logic;
		data_in : IN std_logic_vector(7 downto 0);
		send : IN std_logic;          
		tx_out : OUT std_logic;
		ready : OUT std_logic;
		data_out : OUT std_logic_vector(7 downto 0);
		receive : OUT std_logic;
		uart_active : OUT std_logic
		);
	END COMPONENT;

  signal data_in   : std_logic_vector (7 downto 0);
  --signal data_out  : std_logic_vector (7 downto 0);
  signal in_trg		 : std_logic;

  signal state		: std_logic_vector (3 downto 0);
  signal wen    	: std_logic;

begin

	Inst_simple_uart: simple_uart PORT MAP(
		clk 				=> clk,
		res 				=> res,
		--baudrate 		=> X"01b5",
		baudrate 		=> X"801b",			-- baudrate is 50Mhz / 27 = 1.851 Mhz = 16 * 115200 bd
		rx_in 			=> rx_in,
		tx_out 			=> tx_out,
		data_in 		=> dp_in,
		send 				=> wen,
		ready 			=> open,
		data_out 		=> data_in,
		receive 		=> in_trg,
		uart_active => open
	);




  process (clk,res)
	begin
    if res = '1' then
			dp_trg 		<= '0';
			dp_sel 		<= '1';
			wen 			<= '0';
			state  		<= (others => '0');
			dp_out 		<= (others => '0');

    elsif clk'event and clk = '1' then

			if in_trg = '1' then
			  case state is
		      when X"0" => 	if data_in = X"1b" then
                         	state 		<= X"1";
                         	dp_trg 		<= '0';
			     							else 
                         	dp_out 		<= data_in;
                         	dp_trg 		<= '1';	
			     							end if;
			    when X"1" => 	if data_in = X"1b" then
						 							dp_out 		<=  X"1b";
		                      dp_trg 		<= '1';
		                    elsif data_in = X"00" then
		                      dp_sel 		<= '0';
		                    elsif data_in = X"01" then
		                      dp_sel 		<= '1';
		                    elsif data_in = X"02" then
		                      wen 			<= '1';
		                    elsif data_in = X"03" then
												  dp_trg 		<= '1';
		                      wen 			<= '1';
		                    end if;	
			   		     				state <= X"0";
				  when others => NULL;				
	  		end case;
      else
        dp_trg 	<= '0';
        wen 		<= '0';
      end if;
    end if;

  end process;


end behave;