library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity sram_access is
		port (clk 			: in		std_logic;
					res 			: in 		std_logic;

					-- sram seite (fr 1 sram)

					sri_adr		: out 	std_logic_vector (17 downto 0);
					sri_data	: inout std_logic_vector (15 downto 0);
					sri_cs1		: out		std_logic;
					sri_cs2		: out		std_logic;
					sri_oe		: out		std_logic;
					sri_ub		: out		std_logic;
					sri_lb		: out		std_logic;
					sri_we		: out		std_logic;

					-- fpga seite

					ri_adr		: in  	std_logic_vector (17 downto 0);
					ri_din		: in    std_logic_vector (15 downto 0);
					ri_dout		: out 	std_logic_vector (15 downto 0);
					ri_rw			: in		std_logic;
					ri_trg		: in		std_logic;
					ri_rdy		: out		std_logic
				 );
end sram_access;

architecture Behavioral of sram_access is

	type ram_states is (rs_idle, 
											rs_write, 
											rs_write_01, 
											--rs_write_02, 
											rs_read, 
											rs_read_01, 
											rs_read_02);

	signal ram_state : ram_states;

begin

	process (clk, res, 
	         sri_Data,
	         ram_state, ri_adr, ri_din, ri_rw, ri_trg)
	begin
		sri_ub 	<= '0';
		sri_lb 	<= '0';
		sri_adr <= ri_adr;

		if res = '1' then

			ram_state <= rs_idle;			
			sri_data 	<= (others => 'Z');
			sri_oe 		<= '1';
			sri_cs1 	<= '1';
			sri_cs2		<= '1';
			sri_we 	 	<= '1';
			ri_dout 	<= (others => '0');
      ri_rdy 		<= '0';

		elsif clk'event and clk = '1' then

			case ram_state is
				when rs_idle 			=>  if ri_trg = '1' then
																if ri_rw = '0' then
																	sri_data 	<= (others => 'Z');
																	ram_state <= rs_write;
																	sri_cs1 	<= '1';
																	sri_oe  	<= '1';
																	sri_we  	<= '0';
      														ri_rdy 		<= '0';
																else
																	ram_state <= rs_read;
																	sri_data 	<= (others => 'Z');
																	sri_cs1 	<= '1';
																	sri_oe  	<= '1';
																	sri_we  	<= '1';
														      ri_rdy 		<= '0';
																end if;
															else
																sri_data <= (others => 'Z');
																sri_cs1  		<= '1';
      													ri_rdy 			<= '1';
															end if;

				when rs_write		 	=>	
															sri_data 	<= ri_din;
															sri_cs1 	<= '0';
															sri_oe  	<= '1';
															sri_we  	<= '0';
															ri_rdy 		<= '0';
															ram_state <= rs_write_01;

				when rs_write_01 	=>
															sri_cs1 	<= '1';
															sri_oe  	<= '1';
															sri_we  	<= '0';
															ri_rdy 		<= '0';
															ram_state <= rs_idle;



				when rs_read 			=>
															sri_data 	<= (others => 'Z');
															sri_cs1 	<= '0';
															sri_oe  	<= '0';
															sri_we  	<= '1';
															ri_rdy 		<= '0';
															ram_state <= rs_read_01;

				when rs_read_01 	=>
                              ri_dout 	<= sri_data;
															sri_cs1  	<= '0';
															sri_oe   	<= '0';
															sri_we   	<= '1';
															ri_rdy 		<= '0';
															ram_state <= rs_read_02;

				when rs_read_02 	=>
															sri_cs1  	<= '1';
															sri_oe   	<= '1';
															sri_we   	<= '1';
															ri_rdy 		<= '0';
															ram_state <= rs_idle;

				when others 			=> 	ram_state <= rs_idle;
															ri_rdy 		<= '0';
			end case;

		end if;


	end process;

end Behavioral;
