-----------------------------------------------------------------------------
--
--	SEG7MUX_V2.VHDL
--
--	Implements the 7Segment-Display Multiplexer.
--  Allws to show 16 Bit Hexadecimal Values as well as 32 Bit Segment Values
--  (for accessing all segments individually)
--
--	Author 	: Rene Bllhoff
--  Created : around May 2006
--
--	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
-----------------------------------------------------------------------------
--
--  This source file may be used and distributed without
--  restriction provided that this copyright statement is not
--  removed from the file and that any derivative work contains
--  the original copyright notice and the associated disclaimer.
--
--  This program is free software; you can redistribute it and/or
--  modify it under the terms of the GNU General Public License
--  as published by the Free Software Foundation; either version 2
--  of the License, or (at your option) any later version.
--										
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
--  General Public License for more details.
--
--  You should have received a copy of the GNU General Public
--  License along with this source; if not, download it	from
--  http://www.gnu.org/licenses/gpl.html	
--
-----------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity seg7mux_v2 is
  port (
		clk   		: in  std_logic;
		clk1k   	: in 	std_logic;
		res				:	in	std_logic;
		use_hex 	: in  std_logic;
		use_dots  : in  std_logic;
		value16		: in 	std_logic_vector (15 downto 0);
		value32 	: in 	std_logic_vector (31 downto 0);
		dots4			: in  std_logic_vector ( 3 downto 0);
		segout  	: out	std_logic_vector ( 7 downto 0);
		digout  	: out	std_logic_vector ( 3 downto 0)
	);
end seg7mux_v2;

architecture Behavioral of seg7mux_v2 is

  signal 	strobe 	 : std_logic_vector (1 downto 0);
  signal 	val4   	 : std_logic_vector (3 downto 0);
	signal  seg8     : std_logic_vector (7 downto 0);
	signal	dp			 : std_logic;
	signal	clk1k_z1 : std_logic;

begin
  process (clk1k, res, value16, value32, use_hex, use_dots, dots4, strobe)
	begin
		if res = '1' then
			strobe 		<= (others => '0');
			val4	 		<= (others => '0');
			segout 		<= (others => '1');
			digout 		<= (others => '1');
			seg8   		<= (others => '1');
			dp				<= '1';
			clk1k_z1 	<= '0';
		
		elsif clk'event and clk = '1' then
		  clk1k_z1 <= clk1k;
		
			if clk1k_z1 = '0' and clk1k = '1' then
			  strobe <= strobe + '1';
			end if;

		end if;

		case strobe is
		  when "00" 	=> seg8 <= not (value32 (31 downto 24)); val4 <= value16 ( 3 downto  0); 	digout <= "1110";	dp <= not dots4 (0);
			when "01"	 	=> seg8 <= not (value32 (23 downto 16)); val4 <= value16 ( 7 downto  4); 	digout <= "1101";	dp <= not dots4 (1);
			when "10" 	=> seg8 <= not (value32 (15 downto  8)); val4 <= value16 (11 downto  8); 	digout <= "1011";	dp <= not dots4 (2);
			when "11" 	=> seg8 <= not (value32 ( 7 downto  0)); val4	<= value16 (15 downto 12); 	digout <= "0111";	dp <= not dots4 (3);
			when others	=> seg8 <= "11111111"; 									 val4	<= "0000"; 								 	digout <= "1111";	dp <= '1';
		end case;

		if use_hex = '1' then
			case val4 is
			  when "0000" => segout <= "0000001" & dp;	 	-- 0
				when "0001" => segout <= "1001111" & dp;	 	-- 1
				when "0010" => segout <= "0010010" & dp;		-- 2
				when "0011" => segout <= "0000110" & dp;		-- 3
			  when "0100" => segout <= "1001100" & dp;		-- 4
				when "0101" => segout <= "0100100" & dp;		-- 5
				when "0110" => segout <= "0100000" & dp;		-- 6
				when "0111" => segout <= "0001111" & dp;		-- 7
			  when "1000" => segout <= "0000000" & dp;		-- 8
				when "1001" => segout <= "0000100" & dp;		-- 9
				when "1010" => segout <= "0001000" & dp;		-- a
				when "1011" => segout <= "1100000" & dp;		-- b
			  when "1100" => segout <= "0110001" & dp;		-- c
				when "1101" => segout <= "1000010" & dp;		-- d
				when "1110" => segout <= "0110000" & dp;		-- e
				when "1111" => segout <= "0111000" & dp;		-- f
				when others => segout <= "1111111" & dp;		-- 0
			end case;
		else
			if use_dots = '1' then
			  segout <= seg8 (7 downto 1) & dp;
			else
				segout <= seg8;
			end if;

		end if;

	end process;

end Behavioral;
